
/*
  CoMinder SpaceStat Installation - Packages
  ver.: 0.53
  2011.12.28: 0.50: ACE: S: DBA_SEGMENTS only support 
  2012.02.16: 0.51: ACE: F: DBA_DATA_FILES, DBA_TEMP_FILES support
  2012.06.27: 0.53: ACE: S=>DBS,F=>DBF,DBF: added support of redologs and controlfiles (w/o size)
       
*/


/* snapshot for segments */
CREATE OR REPLACE PROCEDURE SPACESTAT.SPS_SNAP_SEGMENTS(p_snap_nextval in number, p_dbid in number) IS
BEGIN
   INSERT INTO SPACESTAT.SPS_SNAP_DATA(SNAP_ID,DBID,SNAP_TYPE,OWNER,SEGMENT_TYPE,SEGMENT_NAME,PARTITION_NAME,TABLESPACE_NAME,BYTES,BLOCKS,EXTENTS,BUFFER_POOL)
        SELECT p_snap_nextval,p_dbid,'DBS',OWNER,SEGMENT_TYPE,SEGMENT_NAME,PARTITION_NAME,TABLESPACE_NAME,BYTES,BLOCKS,EXTENTS,BUFFER_POOL 
         FROM dba_segments; 
   -- update info about table name for indexes
   UPDATE SPACESTAT.SPS_SNAP_DATA O
          set (O.MASTER_SEGMENT_NAME,O.SEGMENT_SUB_TYPE)=(select I.TABLE_NAME,I.INDEX_TYPE 
               from dba_indexes I 
              where O.OWNER=I.OWNER 
                and O.SEGMENT_NAME=I.INDEX_NAME
              )
         where O.SEGMENT_TYPE='INDEX'
           and O.SNAP_ID=p_snap_nextval
           and O.DBID=p_dbid
           and O.SNAP_TYPE='DBS';
END;
/

/* snapshot for db-files */
CREATE OR REPLACE PROCEDURE SPACESTAT.SPS_SNAP_FILES(p_snap_nextval in number, p_dbid in number) IS
BEGIN
  -- data-files
   INSERT INTO SPACESTAT.SPS_SNAP_DATA(SNAP_ID,DBID,SNAP_TYPE,OWNER,SEGMENT_TYPE,SEGMENT_NAME,PARTITION_NAME,TABLESPACE_NAME,BYTES,BLOCKS,EXTENTS,BUFFER_POOL)
        SELECT p_snap_nextval,p_dbid,'DBF','DATABASE','DATA_FILE',FILE_NAME,'',TABLESPACE_NAME,BYTES,BLOCKS,NULL,NULL 
         FROM dba_data_files;
  -- temp-files
   INSERT INTO SPACESTAT.SPS_SNAP_DATA(SNAP_ID,DBID,SNAP_TYPE,OWNER,SEGMENT_TYPE,SEGMENT_NAME,PARTITION_NAME,TABLESPACE_NAME,BYTES,BLOCKS,EXTENTS,BUFFER_POOL)
        SELECT p_snap_nextval,p_dbid,'DBF','DATABASE','TEMP_FILE',FILE_NAME,'',TABLESPACE_NAME,BYTES,BLOCKS,NULL,NULL 
         FROM dba_temp_files; 
  -- redo-logs
   INSERT INTO SPACESTAT.SPS_SNAP_DATA(SNAP_ID,DBID,SNAP_TYPE,OWNER,SEGMENT_TYPE,SEGMENT_NAME,PARTITION_NAME,TABLESPACE_NAME,BYTES,BLOCKS,EXTENTS,BUFFER_POOL)
        SELECT p_snap_nextval,p_dbid,'DBF','DATABASE','REDOLOG_FILE',LF.MEMBER,'','*GROUP.'||LG.GROUP#||'*',LG.BYTES,1,NULL,NULL 
         FROM v$logfile LF inner join v$log LG on (LF.GROUP#=LG.GROUP#); 
  -- control-files (size not calculated because size info not kept in DB)
   INSERT INTO SPACESTAT.SPS_SNAP_DATA(SNAP_ID,DBID,SNAP_TYPE,OWNER,SEGMENT_TYPE,SEGMENT_NAME,PARTITION_NAME,TABLESPACE_NAME,BYTES,BLOCKS,EXTENTS,BUFFER_POOL)
        SELECT p_snap_nextval,p_dbid,'DBF','DATABASE','CONTROL_FILE',NAME,'','*CONTROLFILES*',1,1,NULL,NULL 
         FROM v$controlfile; 

END;
/

/* main snapshoting procedure */
CREATE OR REPLACE PROCEDURE SPACESTAT.SPS_MAKE_SNAP(p_SNAP_TYPE IN varchar2) IS
   v_dbid number;
   v_dbname varchar(9);
   v_snap_type varchar2(3);
   v_snap_nextval number;
   v_status varchar(10);

BEGIN
  v_snap_type:=UPPER(substr(trim(p_SNAP_TYPE),1,3));
  
  if (v_snap_type = 'ALL') then
     -- just do 2 recursive calls
     -- files first, because it faster in most cases and use less space
     SPACESTAT.SPS_MAKE_SNAP('DBF');
     SPACESTAT.SPS_MAKE_SNAP('DBS');
  else
    -- do single snap operations
    SELECT NAME, DBID INTO v_dbname,v_dbid  FROM V$DATABASE;
    SELECT SPACESTAT.SPS_SNAP_ID_SEQ.NEXTVAL INTO v_snap_nextval FROM dual;
    INSERT INTO SPACESTAT.SPS_SNAPS(snap_id,db_name,dbid,SNAP_TYPE,STATUS,start_time) VALUES(v_snap_nextval, v_dbname, v_dbid, v_snap_type, 'RUN', SYSDATE);
     
    CASE v_snap_type
      WHEN 'DBS' THEN
        SPACESTAT.SPS_SNAP_SEGMENTS(v_snap_nextval,v_dbid);
        v_status:='OK';
      WHEN 'DBF' THEN
        SPACESTAT.SPS_SNAP_FILES(v_snap_nextval,v_dbid);
        v_status:='OK';
      ELSE
        raise_application_error(-20001, 'Sorry, segments snapshot of type '||v_snap_type||' is not supported');
        v_status:='NOK';
    END CASE;
     --
     UPDATE SPACESTAT.SPS_SNAPS 
        SET end_time = SYSDATE, STATUS=v_status 
      WHERE SNAP_ID = v_snap_nextval 
        and DBID=v_dbid
        and SNAP_TYPE=v_snap_type;
     COMMIT;
   end if;  
END;
/


--- The End ----