
/*
  CoMinder SpaceStat Installation - Tables, Indexes, Constraints
  ver.: 0.53
*/

create table SPACESTAT.SPS_VERSION(
  module_name varchar2(30),
  module_info varchar2(255), 
  module_version varchar(8),
  status varchar2(7)
  install_ts DATE, 
  upgrade_ts DATE
  
) tablespace SPACESTAT
/

insert into SPACESTAT.SPS_VERSION(module_name,module_info,module_version,status,install_ts,upgrade_ts)
  values('SPS_ORADB','SpaceStat for Oracle DB','0.53','install',sysdate,sysdate);
commit;   

-- sequence
drop SEQUENCE SPACESTAT.SPS_SNAP_ID_SEQ;

CREATE SEQUENCE SPACESTAT.SPS_SNAP_ID_SEQ
NOCACHE 
start with 1
increment by 1 
MAXVALUE 9999999
/

-- table with snaps info
drop table SPACESTAT.SPS_SNAPS;

create table SPACESTAT.SPS_SNAPS(
  snap_id NUMBER(8) NOT NULL, 
  dbid NUMBER       NOT NULL,
  snap_type VARCHAR2(3) NOT NULL,
  db_name VARCHAR2(30)  NOT NULL,
  start_time DATE   NOT NULL, 
  end_time DATE,
  status VARCHAR2(3) NO NULL
) tablespace SPACESTAT
/

---- UK-PK-Index
CREATE Unique INDEX SPACESTAT.SPS_SNAPS_PK ON SPACESTAT.SPS_SNAPS
   (  SNAP_ID ASC, 
      DBID ASC, 
      SNAP_TYPE ASC) 
  TABLESPACE SPACESTAT
/

-- add PK 
alter table SPACESTAT.SPS_SNAPS add constraint SPS_SNAPS_PK PRIMARY KEY
(
  SNAP_ID, DBID, SNAP_TYPE
)
USING INDEX
tablespace SPACESTAT;

-- table with target segments used space information 
drop table SPACESTAT.SPS_SNAP_DATA;

create table SPACESTAT.SPS_SNAP_DATA(
  snap_id NUMBER(8) NOT NULL, 
  dbid NUMBER       NOT NULL,
  snap_type VARCHAR2(3) NOT NULL,
  OWNER VARCHAR2(30),
  SEGMENT_TYPE  VARCHAR2(18),
  SEGMENT_NAME VARCHAR2(256),
  SEGMENT_SUB_TYPE  VARCHAR2(27),
  MASTER_SEGMENT_TYPE  VARCHAR2(30),  -- table in most cases 
  MASTER_SEGMENT_NAME VARCHAR2(256),  -- table name in most cases - used for indexes  
  PARTITION_NAME VARCHAR2(30),
  TABLESPACE_NAME VARCHAR2(30),
  BYTES     NUMBER(16,0),
  BLOCKS     NUMBER(16,0),
  EXTENTS     NUMBER(16,0),
  BUFFER_POOL  VARCHAR2(7),
  BUSINESS_MODULE VARCHAR2(10),  -- reserved for custom use
  DATA_CLASS      VARCHAR2(5),   -- reserved for custom use: FACT,FOLTP,FDWH,DICT,DEBUG,AUDIT,ANY
  SIZE_CLASS      VARCHAR2(3)    -- reserved for custom use: SML, AVG, BIG
) TABLESPACE SPACESTAT 
/

---- UK-PK-Index
CREATE UNIQUE INDEX SPACESTAT.SPS_SNAP_DATA_PK ON SPACESTAT.SPS_SNAP_DATA
  (
    SNAP_ID,
    DBID,
    SNAP_TYPE,
    OWNER,
    SEGMENT_TYPE,
    SEGMENT_NAME
  ) tablespace SPACESTAT;
  
alter table SPACESTAT.SPS_SNAP_DATA 
add constraint SPS_SNAP_DATA_PK primary key
(
  SNAP_ID,
  DBID,
  SNAP_TYPE,
  OWNER,
  SEGMENT_TYPE,
  SEGMENT_NAME
)
USING INDEX
tablespace SPACESTAT
/ 

-- add FK constraint to snaps
alter table SPACESTAT.SPS_SNAP_DATA 
add constraint SPS_SNAP_DATA_SNAP_FK foreign key 
(snap_id, dbid, snap_type) references SPACESTAT.SPS_SNAPS_PK on delete cascade
/


--- The End ----
